'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function noop() { }
function assign(tar, src) {
    // @ts-ignore
    for (const k in src)
        tar[k] = src[k];
    return tar;
}
function run(fn) {
    return fn();
}
function blank_object() {
    return Object.create(null);
}
function run_all(fns) {
    fns.forEach(run);
}
function is_function(thing) {
    return typeof thing === 'function';
}
function safe_not_equal(a, b) {
    return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
}
function is_empty(obj) {
    return Object.keys(obj).length === 0;
}
function create_slot(definition, ctx, $$scope, fn) {
    if (definition) {
        const slot_ctx = get_slot_context(definition, ctx, $$scope, fn);
        return definition[0](slot_ctx);
    }
}
function get_slot_context(definition, ctx, $$scope, fn) {
    return definition[1] && fn
        ? assign($$scope.ctx.slice(), definition[1](fn(ctx)))
        : $$scope.ctx;
}
function get_slot_changes(definition, $$scope, dirty, fn) {
    if (definition[2] && fn) {
        const lets = definition[2](fn(dirty));
        if ($$scope.dirty === undefined) {
            return lets;
        }
        if (typeof lets === 'object') {
            const merged = [];
            const len = Math.max($$scope.dirty.length, lets.length);
            for (let i = 0; i < len; i += 1) {
                merged[i] = $$scope.dirty[i] | lets[i];
            }
            return merged;
        }
        return $$scope.dirty | lets;
    }
    return $$scope.dirty;
}
function update_slot_base(slot, slot_definition, ctx, $$scope, slot_changes, get_slot_context_fn) {
    if (slot_changes) {
        const slot_context = get_slot_context(slot_definition, ctx, $$scope, get_slot_context_fn);
        slot.p(slot_context, slot_changes);
    }
}
function get_all_dirty_from_scope($$scope) {
    if ($$scope.ctx.length > 32) {
        const dirty = [];
        const length = $$scope.ctx.length / 32;
        for (let i = 0; i < length; i++) {
            dirty[i] = -1;
        }
        return dirty;
    }
    return -1;
}
function detach(node) {
    node.parentNode.removeChild(node);
}
function children(element) {
    return Array.from(element.childNodes);
}
function custom_event(type, detail, bubbles = false) {
    const e = document.createEvent('CustomEvent');
    e.initCustomEvent(type, bubbles, false, detail);
    return e;
}

let current_component;
function set_current_component(component) {
    current_component = component;
}
function get_current_component() {
    if (!current_component)
        throw new Error('Function called outside component initialization');
    return current_component;
}
function createEventDispatcher() {
    const component = get_current_component();
    return (type, detail) => {
        const callbacks = component.$$.callbacks[type];
        if (callbacks) {
            // TODO are there situations where events could be dispatched
            // in a server (non-DOM) environment?
            const event = custom_event(type, detail);
            callbacks.slice().forEach(fn => {
                fn.call(component, event);
            });
        }
    };
}

const dirty_components = [];
const binding_callbacks = [];
const render_callbacks = [];
const flush_callbacks = [];
const resolved_promise = Promise.resolve();
let update_scheduled = false;
function schedule_update() {
    if (!update_scheduled) {
        update_scheduled = true;
        resolved_promise.then(flush);
    }
}
function add_render_callback(fn) {
    render_callbacks.push(fn);
}
function add_flush_callback(fn) {
    flush_callbacks.push(fn);
}
let flushing = false;
const seen_callbacks = new Set();
function flush() {
    if (flushing)
        return;
    flushing = true;
    do {
        // first, call beforeUpdate functions
        // and update components
        for (let i = 0; i < dirty_components.length; i += 1) {
            const component = dirty_components[i];
            set_current_component(component);
            update(component.$$);
        }
        set_current_component(null);
        dirty_components.length = 0;
        while (binding_callbacks.length)
            binding_callbacks.pop()();
        // then, once components are updated, call
        // afterUpdate functions. This may cause
        // subsequent updates...
        for (let i = 0; i < render_callbacks.length; i += 1) {
            const callback = render_callbacks[i];
            if (!seen_callbacks.has(callback)) {
                // ...so guard against infinite loops
                seen_callbacks.add(callback);
                callback();
            }
        }
        render_callbacks.length = 0;
    } while (dirty_components.length);
    while (flush_callbacks.length) {
        flush_callbacks.pop()();
    }
    update_scheduled = false;
    flushing = false;
    seen_callbacks.clear();
}
function update($$) {
    if ($$.fragment !== null) {
        $$.update();
        run_all($$.before_update);
        const dirty = $$.dirty;
        $$.dirty = [-1];
        $$.fragment && $$.fragment.p($$.ctx, dirty);
        $$.after_update.forEach(add_render_callback);
    }
}
const outroing = new Set();
let outros;
function transition_in(block, local) {
    if (block && block.i) {
        outroing.delete(block);
        block.i(local);
    }
}
function transition_out(block, local, detach, callback) {
    if (block && block.o) {
        if (outroing.has(block))
            return;
        outroing.add(block);
        outros.c.push(() => {
            outroing.delete(block);
            if (callback) {
                if (detach)
                    block.d(1);
                callback();
            }
        });
        block.o(local);
    }
}

function bind(component, name, callback) {
    const index = component.$$.props[name];
    if (index !== undefined) {
        component.$$.bound[index] = callback;
        callback(component.$$.ctx[index]);
    }
}
function create_component(block) {
    block && block.c();
}
function mount_component(component, target, anchor, customElement) {
    const { fragment, on_mount, on_destroy, after_update } = component.$$;
    fragment && fragment.m(target, anchor);
    if (!customElement) {
        // onMount happens before the initial afterUpdate
        add_render_callback(() => {
            const new_on_destroy = on_mount.map(run).filter(is_function);
            if (on_destroy) {
                on_destroy.push(...new_on_destroy);
            }
            else {
                // Edge case - component was destroyed immediately,
                // most likely as a result of a binding initialising
                run_all(new_on_destroy);
            }
            component.$$.on_mount = [];
        });
    }
    after_update.forEach(add_render_callback);
}
function destroy_component(component, detaching) {
    const $$ = component.$$;
    if ($$.fragment !== null) {
        run_all($$.on_destroy);
        $$.fragment && $$.fragment.d(detaching);
        // TODO null out other refs, including component.$$ (but need to
        // preserve final state?)
        $$.on_destroy = $$.fragment = null;
        $$.ctx = [];
    }
}
function make_dirty(component, i) {
    if (component.$$.dirty[0] === -1) {
        dirty_components.push(component);
        schedule_update();
        component.$$.dirty.fill(0);
    }
    component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
}
function init(component, options, instance, create_fragment, not_equal, props, append_styles, dirty = [-1]) {
    const parent_component = current_component;
    set_current_component(component);
    const $$ = component.$$ = {
        fragment: null,
        ctx: null,
        // state
        props,
        update: noop,
        not_equal,
        bound: blank_object(),
        // lifecycle
        on_mount: [],
        on_destroy: [],
        on_disconnect: [],
        before_update: [],
        after_update: [],
        context: new Map(options.context || (parent_component ? parent_component.$$.context : [])),
        // everything else
        callbacks: blank_object(),
        dirty,
        skip_bound: false,
        root: options.target || parent_component.$$.root
    };
    append_styles && append_styles($$.root);
    let ready = false;
    $$.ctx = instance
        ? instance(component, options.props || {}, (i, ret, ...rest) => {
            const value = rest.length ? rest[0] : ret;
            if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                if (!$$.skip_bound && $$.bound[i])
                    $$.bound[i](value);
                if (ready)
                    make_dirty(component, i);
            }
            return ret;
        })
        : [];
    $$.update();
    ready = true;
    run_all($$.before_update);
    // `false` as a special case of no DOM component
    $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
    if (options.target) {
        if (options.hydrate) {
            const nodes = children(options.target);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.l(nodes);
            nodes.forEach(detach);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.c();
        }
        if (options.intro)
            transition_in(component.$$.fragment);
        mount_component(component, options.target, options.anchor, options.customElement);
        flush();
    }
    set_current_component(parent_component);
}
/**
 * Base class for Svelte components. Used when dev=false.
 */
class SvelteComponent {
    $destroy() {
        destroy_component(this, 1);
        this.$destroy = noop;
    }
    $on(type, callback) {
        const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
        callbacks.push(callback);
        return () => {
            const index = callbacks.indexOf(callback);
            if (index !== -1)
                callbacks.splice(index, 1);
        };
    }
    $set($$props) {
        if (this.$$set && !is_empty($$props)) {
            this.$$.skip_bound = true;
            this.$$set($$props);
            this.$$.skip_bound = false;
        }
    }
}

function dlv(t,e,l,n,r){for(e=e.split?e.split("."):e,n=0;n<e.length;n++)t=t?t[e[n]]:r;return t===r?l:t}

/*

Note on escaping order, from RFC6901:

> Evaluation of each reference token begins by decoding any escaped
> character sequence.  This is performed by first transforming any
> occurrence of the sequence '~1' to '/', and then transforming any
> occurrence of the sequence '~0' to '~'.  By performing the
> substitutions in this order, an implementation avoids the error of
> turning '~01' first into '~1' and then into '/', which would be
> incorrect (the string '~01' correctly becomes '~1' after
> transformation).

*/

/**
 * Convert a JSON Pointer into a list of unescaped tokens, e.g. `/foo/bar~1biz` to `['foo','bar/biz']`.
 * @type {import("../index").toTokens}
 */
const toTokens = function (path) {
	[ , ...path ] = path.split('/');
	let segments = [];
	for (let segment of path) {
		segments.push(segment.replaceAll('~1', '/').replaceAll('~0', '~'));
	}
	return segments
};

/**
 * @param {String|Array<String>} input
 * @returns {Array<String>}
 */
const makeConsistent = input => input.split ? toTokens(input) : input;

/**
 * Access a property by JSON Pointer, or by an array of property tokens.
 * @type {import("../index").get}
 */
const get = function (obj, path) {
	return dlv(obj, makeConsistent(path))
};

var justDebounceIt = debounce;

function debounce(fn, wait, callFirst) {
  var timeout = null;
  var debouncedFn = null;

  var clear = function() {
    if (timeout) {
      clearTimeout(timeout);

      debouncedFn = null;
      timeout = null;
    }
  };

  var flush = function() {
    var call = debouncedFn;
    clear();

    if (call) {
      call();
    }
  };

  var debounceWrapper = function() {
    if (!wait) {
      return fn.apply(this, arguments);
    }

    var context = this;
    var args = arguments;
    var callNow = callFirst && !timeout;
    clear();

    debouncedFn = function() {
      fn.apply(context, args);
    };

    timeout = setTimeout(function() {
      timeout = null;

      if (!callNow) {
        var call = debouncedFn;
        debouncedFn = null;

        return call();
      }
    }, wait);

    if (callNow) {
      return debouncedFn();
    }
  };

  debounceWrapper.cancel = clear;
  debounceWrapper.flush = flush;

  return debounceWrapper;
}

var justDiff = {
  diff: diff,
  jsonPatchPathConverter: jsonPatchPathConverter,
};

/*
  const obj1 = {a: 4, b: 5};
  const obj2 = {a: 3, b: 5};
  const obj3 = {a: 4, c: 5};

  diff(obj1, obj2);
  [
    { "op": "replace", "path": ['a'], "value": 3 }
  ]

  diff(obj2, obj3);
  [
    { "op": "remove", "path": ['b'] },
    { "op": "replace", "path": ['a'], "value": 4 }
    { "op": "add", "path": ['c'], "value": 5 }
  ]

  // using converter to generate jsPatch standard paths
  // see http://jsonpatch.com
  import {diff, jsonPatchPathConverter} from 'just-diff'
  diff(obj1, obj2, jsonPatchPathConverter);
  [
    { "op": "replace", "path": '/a', "value": 3 }
  ]

  diff(obj2, obj3, jsonPatchPathConverter);
  [
    { "op": "remove", "path": '/b' },
    { "op": "replace", "path": '/a', "value": 4 }
    { "op": "add", "path": '/c', "value": 5 }
  ]

  // arrays
  const obj4 = {a: 4, b: [1, 2, 3]};
  const obj5 = {a: 3, b: [1, 2, 4]};
  const obj6 = {a: 3, b: [1, 2, 4, 5]};

  diff(obj4, obj5);
  [
    { "op": "replace", "path": ['a'], "value": 3 }
    { "op": "replace", "path": ['b', 2], "value": 4 }
  ]

  diff(obj5, obj6);
  [
    { "op": "add", "path": ['b', 3], "value": 5 }
  ]

  // nested paths
  const obj7 = {a: 4, b: {c: 3}};
  const obj8 = {a: 4, b: {c: 4}};
  const obj9 = {a: 5, b: {d: 4}};

  diff(obj7, obj8);
  [
    { "op": "replace", "path": ['b', 'c'], "value": 4 }
  ]

  diff(obj8, obj9);
  [
    { "op": "replace", "path": ['a'], "value": 5 }
    { "op": "remove", "path": ['b', 'c']}
    { "op": "add", "path": ['b', 'd'], "value": 4 }
  ]
*/

function diff(obj1, obj2, pathConverter) {
  if (!obj1 || typeof obj1 != 'object' || !obj2 || typeof obj2 != 'object') {
    throw new Error('both arguments must be objects or arrays');
  }

  pathConverter ||
    (pathConverter = function(arr) {
      return arr;
    });

  function getDiff(obj1, obj2, basePath, diffs) {
    var obj1Keys = Object.keys(obj1);
    var obj1KeysLength = obj1Keys.length;
    var obj2Keys = Object.keys(obj2);
    var obj2KeysLength = obj2Keys.length;
    var path;

    for (var i = 0; i < obj1KeysLength; i++) {
      var key = Array.isArray(obj1) ? Number(obj1Keys[i]) : obj1Keys[i];
      if (!(key in obj2)) {
        path = basePath.concat(key);
        diffs.remove.push({
          op: 'remove',
          path: pathConverter(path),
        });
      }
    }

    for (var i = 0; i < obj2KeysLength; i++) {
      var key = Array.isArray(obj2) ? Number(obj2Keys[i]) : obj2Keys[i];
      var obj1AtKey = obj1[key];
      var obj2AtKey = obj2[key];
      if (!(key in obj1)) {
        path = basePath.concat(key);
        var obj2Value = obj2[key];
        diffs.add.push({
          op: 'add',
          path: pathConverter(path),
          value: obj2Value,
        });
      } else if (obj1AtKey !== obj2AtKey) {
        if (
          Object(obj1AtKey) !== obj1AtKey ||
          Object(obj2AtKey) !== obj2AtKey
        ) {
          path = pushReplace(path, basePath, key, diffs, pathConverter, obj2);
        } else {
          if (
            !Object.keys(obj1AtKey).length &&
            !Object.keys(obj2AtKey).length &&
            String(obj1AtKey) != String(obj2AtKey)
          ) {
            path = pushReplace(path, basePath, key, diffs, pathConverter, obj2);
          } else {
            getDiff(obj1[key], obj2[key], basePath.concat(key), diffs);
          }
        }
      }
    }

    return diffs.remove
      .reverse()
      .concat(diffs.replace)
      .concat(diffs.add);
  }
  return getDiff(obj1, obj2, [], {remove: [], replace: [], add: []});
}

function pushReplace(path, basePath, key, diffs, pathConverter, obj2) {
  path = basePath.concat(key);
  diffs.replace.push({
    op: 'replace',
    path: pathConverter(path),
    value: obj2[key],
  });
  return path;
}

function jsonPatchPathConverter(arrayPath) {
  return [''].concat(arrayPath).join('/');
}

/* src/FieldSetter.svelte generated by Svelte v3.43.0 */
const get_default_slot_changes$6 = dirty => ({});
const get_default_slot_context$6 = ctx => ({ set: /*set*/ ctx[0] });

function create_fragment$6(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[6].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[5], get_default_slot_context$6);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, [dirty]) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope*/ 32)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[5],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[5])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[5], dirty, get_default_slot_changes$6),
						get_default_slot_context$6
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

function instance$6($$self, $$props, $$invalidate) {
	let { $$slots: slots = {}, $$scope } = $$props;
	const { diff } = justDiff;
	let { form } = $$props;
	let { id } = $$props;
	let { tokens } = $$props;
	let { debounceMillis } = $$props;

	/*
Because the diff calculation is expensive, we debounce so that e.g. entering text
rapidly won't cause a sudden pile of blocking diff calculations to slow the UI.
 */
	const dispatch = createEventDispatcher();

	const change = justDebounceIt(
		updatedValue => {
			if (!form?.data?.[id]) {
				// TODO this kind of error-appender anywhere there's an error state
				$$invalidate(1, form.errors = form.errors || { other: [], mapped: {} }, form);

				$$invalidate(1, form.errors.other = form.errors.other || [], form);

				if (!form.errors.other.find(e => e.meta?.formErrorId === id)) {
					form.errors.other.push({
						code: 'Incorrectly Generated Form',
						detail: `Field was created but was not found on the form. [id=${id},keypath=/${tokens.join('/')}]`,
						meta: { formErrorId: id }
					});
				}

				return;
			}

			let [k1, k2, k3, k4, k5, k6] = tokens;
			let l = tokens.length;

			/*
The Svelte compiler looks for reassignment as the method to detect whether a
function inside a component is modifying a bound value. Because of this, the
reassignment process can't use the normal shortcut found in e.g. @lukeed/dset,
thus the following method which is limited in depth.
 */
			l > 0 && (form.data[id][k1] ?? $$invalidate(1, form.data[id][k1] = {}, form));

			l > 1 && (form.data[id][k1][k2] ?? $$invalidate(1, form.data[id][k1][k2] = {}, form));
			l > 2 && (form.data[id][k1][k2][k3] ?? $$invalidate(1, form.data[id][k1][k2][k3] = {}, form));
			l > 3 && (form.data[id][k1][k2][k3][k4] ?? $$invalidate(1, form.data[id][k1][k2][k3][k4] = {}, form));
			l > 4 && (form.data[id][k1][k2][k3][k4][k5] ?? $$invalidate(1, form.data[id][k1][k2][k3][k4][k5] = {}, form));
			l > 5 && (form.data[id][k1][k2][k3][k4][k5][k6] ?? $$invalidate(1, form.data[id][k1][k2][k3][k4][k5][k6] = {}, form));
			if (l === 1) $$invalidate(1, form.data[id][k1] = updatedValue, form);
			if (l === 2) $$invalidate(1, form.data[id][k1][k2] = updatedValue, form);
			if (l === 3) $$invalidate(1, form.data[id][k1][k2][k3] = updatedValue, form);
			if (l === 4) $$invalidate(1, form.data[id][k1][k2][k3][k4] = updatedValue, form);
			if (l === 5) $$invalidate(1, form.data[id][k1][k2][k3][k4][k5] = updatedValue, form);
			if (l === 6) $$invalidate(1, form.data[id][k1][k2][k3][k4][k5][k6] = updatedValue, form);
			$$invalidate(1, form.changes[id] = diff(form.original[id] || {}, form.data[id] || {}), form);
			if (!form.changes[id].length) delete form.changes[id];
			if (Object.keys(form.changes).length) $$invalidate(1, form.state = 'changed', form); else $$invalidate(1, form.state = 'unchanged', form);
			dispatch('change', { id, keypath: tokens, value: updatedValue });
		},
		debounceMillis || 15,
		true
	);

	const set = v => {
		// the timeout is used to let the form state settle after a create
		setTimeout(
			() => {
				change(v);
			},
			0
		);
	};

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(1, form = $$props.form);
		if ('id' in $$props) $$invalidate(2, id = $$props.id);
		if ('tokens' in $$props) $$invalidate(3, tokens = $$props.tokens);
		if ('debounceMillis' in $$props) $$invalidate(4, debounceMillis = $$props.debounceMillis);
		if ('$$scope' in $$props) $$invalidate(5, $$scope = $$props.$$scope);
	};

	return [set, form, id, tokens, debounceMillis, $$scope, slots];
}

class FieldSetter extends SvelteComponent {
	constructor(options) {
		super();

		init(this, options, instance$6, create_fragment$6, safe_not_equal, {
			form: 1,
			id: 2,
			tokens: 3,
			debounceMillis: 4,
			set: 0
		});
	}

	get set() {
		return this.$$.ctx[0];
	}
}

/* src/MakeDiff.svelte generated by Svelte v3.43.0 */
const get_default_slot_changes$5 = dirty => ({});
const get_default_slot_context$5 = ctx => ({ makeDiff: /*makeDiff*/ ctx[0] });

function create_fragment$5(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[2].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[1], get_default_slot_context$5);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, [dirty]) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope*/ 2)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[1],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[1])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[1], dirty, get_default_slot_changes$5),
						get_default_slot_context$5
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

function instance$5($$self, $$props, $$invalidate) {
	let { $$slots: slots = {}, $$scope } = $$props;
	const { diff } = justDiff;

	const makeDiff = (form, id) => {
		form.changes[id] = diff(form.original[id] || {}, form.data[id] || {});
		if (!form.changes[id].length) form.changes[id] = undefined;
		if (Object.keys(form.changes).length) form.state = 'unsaved'; else form.state = 'unchanged';
	};

	$$self.$$set = $$props => {
		if ('$$scope' in $$props) $$invalidate(1, $$scope = $$props.$$scope);
	};

	return [makeDiff, $$scope, slots];
}

class MakeDiff extends SvelteComponent {
	constructor(options) {
		super();
		init(this, options, instance$5, create_fragment$5, safe_not_equal, {});
	}
}

/* src/FormCreate.svelte generated by Svelte v3.43.0 */
const get_default_slot_changes$4 = dirty => ({ create: dirty & /*makeDiff*/ 32 });

const get_default_slot_context$4 = ctx => ({
	create: /*create*/ ctx[0](/*makeDiff*/ ctx[5])
});

// (34:0) <MakeDiff let:makeDiff>
function create_default_slot$4(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[2].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[3], get_default_slot_context$4);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, dirty) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope, makeDiff*/ 40)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[3],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[3])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[3], dirty, get_default_slot_changes$4),
						get_default_slot_context$4
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

function create_fragment$4(ctx) {
	let makediff;
	let current;

	makediff = new MakeDiff({
			props: {
				$$slots: {
					default: [
						create_default_slot$4,
						({ makeDiff }) => ({ 5: makeDiff }),
						({ makeDiff }) => makeDiff ? 32 : 0
					]
				},
				$$scope: { ctx }
			}
		});

	return {
		c() {
			create_component(makediff.$$.fragment);
		},
		m(target, anchor) {
			mount_component(makediff, target, anchor);
			current = true;
		},
		p(ctx, [dirty]) {
			const makediff_changes = {};

			if (dirty & /*$$scope, makeDiff*/ 40) {
				makediff_changes.$$scope = { dirty, ctx };
			}

			makediff.$set(makediff_changes);
		},
		i(local) {
			if (current) return;
			transition_in(makediff.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(makediff.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(makediff, detaching);
		}
	};
}

function instance$4($$self, $$props, $$invalidate) {
	let { $$slots: slots = {}, $$scope } = $$props;
	let { form } = $$props;
	const dispatch = createEventDispatcher();

	/**
 * Create a new resource in the Form, with a generated identifier.
 *
 * @type {import('..').create}
 */
	const create = makeDiff => ({ relId, relName, isArray, type, resource }) => {
		if (!form.gidIndex) $$invalidate(1, form.gidIndex = 0, form);
		let id = `${form.gidPrefix || 'GID'}${$$invalidate(1, ++form.gidIndex, form)}${form.gidSuffix || ''}`;
		$$invalidate(1, form.data[id] = { type, id }, form);
		makeDiff(form, id);
		if (!form.data[relId].relationships) $$invalidate(1, form.data[relId].relationships = {}, form);
		if (!form.data[relId].relationships[relName]) $$invalidate(1, form.data[relId].relationships[relName] = {}, form);

		if (isArray) {
			let data = form.data[relId].relationships[relName].data || [];
			data.push({ type, id });
			$$invalidate(1, form.data[relId].relationships[relName].data = data, form);
		} else {
			$$invalidate(1, form.data[relId].relationships[relName].data = { ...resource, type, id }, form);
		}

		makeDiff(form, relId);
		dispatch('create', { relId, relName, isArray, type });
	};

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(1, form = $$props.form);
		if ('$$scope' in $$props) $$invalidate(3, $$scope = $$props.$$scope);
	};

	return [create, form, slots, $$scope];
}

class FormCreate extends SvelteComponent {
	constructor(options) {
		super();
		init(this, options, instance$4, create_fragment$4, safe_not_equal, { form: 1 });
	}
}

/* src/Field.svelte generated by Svelte v3.43.0 */

const get_default_slot_changes$3 = dirty => ({
	create: dirty & /*create*/ 4096,
	set: dirty & /*set*/ 8192,
	value: dirty & /*value*/ 64,
	errors: dirty & /*errors*/ 32,
	disabled: dirty & /*disabled*/ 16
});

const get_default_slot_context$3 = ctx => ({
	create: /*create*/ ctx[12],
	set: /*set*/ ctx[13],
	value: /*value*/ ctx[6],
	errors: /*errors*/ ctx[5],
	disabled: /*disabled*/ ctx[4]
});

// (26:1) <FieldSetter bind:form {id} {tokens} {debounceMillis} let:set>
function create_default_slot_1$2(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[8].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[11], get_default_slot_context$3);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, dirty) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope, create, set, value, errors, disabled*/ 14448)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[11],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[11])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[11], dirty, get_default_slot_changes$3),
						get_default_slot_context$3
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

// (25:0) <FormCreate bind:form let:create>
function create_default_slot$3(ctx) {
	let fieldsetter;
	let updating_form;
	let current;

	function fieldsetter_form_binding(value) {
		/*fieldsetter_form_binding*/ ctx[9](value);
	}

	let fieldsetter_props = {
		id: /*id*/ ctx[1],
		tokens: /*tokens*/ ctx[3],
		debounceMillis: /*debounceMillis*/ ctx[2],
		$$slots: {
			default: [
				create_default_slot_1$2,
				({ set }) => ({ 13: set }),
				({ set }) => set ? 8192 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		fieldsetter_props.form = /*form*/ ctx[0];
	}

	fieldsetter = new FieldSetter({ props: fieldsetter_props });
	binding_callbacks.push(() => bind(fieldsetter, 'form', fieldsetter_form_binding));

	return {
		c() {
			create_component(fieldsetter.$$.fragment);
		},
		m(target, anchor) {
			mount_component(fieldsetter, target, anchor);
			current = true;
		},
		p(ctx, dirty) {
			const fieldsetter_changes = {};
			if (dirty & /*id*/ 2) fieldsetter_changes.id = /*id*/ ctx[1];
			if (dirty & /*tokens*/ 8) fieldsetter_changes.tokens = /*tokens*/ ctx[3];
			if (dirty & /*debounceMillis*/ 4) fieldsetter_changes.debounceMillis = /*debounceMillis*/ ctx[2];

			if (dirty & /*$$scope, create, set, value, errors, disabled*/ 14448) {
				fieldsetter_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				fieldsetter_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			fieldsetter.$set(fieldsetter_changes);
		},
		i(local) {
			if (current) return;
			transition_in(fieldsetter.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(fieldsetter.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(fieldsetter, detaching);
		}
	};
}

function create_fragment$3(ctx) {
	let formcreate;
	let updating_form;
	let current;

	function formcreate_form_binding(value) {
		/*formcreate_form_binding*/ ctx[10](value);
	}

	let formcreate_props = {
		$$slots: {
			default: [
				create_default_slot$3,
				({ create }) => ({ 12: create }),
				({ create }) => create ? 4096 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		formcreate_props.form = /*form*/ ctx[0];
	}

	formcreate = new FormCreate({ props: formcreate_props });
	binding_callbacks.push(() => bind(formcreate, 'form', formcreate_form_binding));

	return {
		c() {
			create_component(formcreate.$$.fragment);
		},
		m(target, anchor) {
			mount_component(formcreate, target, anchor);
			current = true;
		},
		p(ctx, [dirty]) {
			const formcreate_changes = {};

			if (dirty & /*$$scope, id, tokens, debounceMillis, form, create, value, errors, disabled*/ 6271) {
				formcreate_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				formcreate_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			formcreate.$set(formcreate_changes);
		},
		i(local) {
			if (current) return;
			transition_in(formcreate.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(formcreate.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(formcreate, detaching);
		}
	};
}

function instance$3($$self, $$props, $$invalidate) {
	let tokens;
	let value;
	let errors;
	let disabled;
	let { $$slots: slots = {}, $$scope } = $$props;
	let { form } = $$props;
	let { id } = $$props;
	let { keypath } = $$props;
	let { debounceMillis } = $$props;

	function fieldsetter_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	function formcreate_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(0, form = $$props.form);
		if ('id' in $$props) $$invalidate(1, id = $$props.id);
		if ('keypath' in $$props) $$invalidate(7, keypath = $$props.keypath);
		if ('debounceMillis' in $$props) $$invalidate(2, debounceMillis = $$props.debounceMillis);
		if ('$$scope' in $$props) $$invalidate(11, $$scope = $$props.$$scope);
	};

	$$self.$$.update = () => {
		if ($$self.$$.dirty & /*keypath*/ 128) {
			$$invalidate(3, tokens = keypath.split ? toTokens(keypath) : keypath);
		}

		if ($$self.$$.dirty & /*form, id, tokens*/ 11) {
			$$invalidate(6, value = get(form.data[id], tokens) || '');
		}

		if ($$self.$$.dirty & /*form, id, tokens*/ 11) {
			$$invalidate(5, errors = get(form, ['errors', 'mapped', id, ...tokens]) || []);
		}

		if ($$self.$$.dirty & /*form*/ 1) {
			$$invalidate(4, disabled = !form.state || ['saving', 'loading'].includes(form.state));
		}
	};

	return [
		form,
		id,
		debounceMillis,
		tokens,
		disabled,
		errors,
		value,
		keypath,
		slots,
		fieldsetter_form_binding,
		formcreate_form_binding,
		$$scope
	];
}

class Field extends SvelteComponent {
	constructor(options) {
		super();

		init(this, options, instance$3, create_fragment$3, safe_not_equal, {
			form: 0,
			id: 1,
			keypath: 7,
			debounceMillis: 2
		});
	}
}

/* src/FormRemove.svelte generated by Svelte v3.43.0 */
const get_default_slot_changes$2 = dirty => ({ remove: dirty & /*makeDiff*/ 32 });

const get_default_slot_context$2 = ctx => ({
	remove: /*remove*/ ctx[0](/*makeDiff*/ ctx[5])
});

// (49:0) <MakeDiff let:makeDiff>
function create_default_slot$2(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[2].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[3], get_default_slot_context$2);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, dirty) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope, makeDiff*/ 40)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[3],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[3])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[3], dirty, get_default_slot_changes$2),
						get_default_slot_context$2
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

function create_fragment$2(ctx) {
	let makediff;
	let current;

	makediff = new MakeDiff({
			props: {
				$$slots: {
					default: [
						create_default_slot$2,
						({ makeDiff }) => ({ 5: makeDiff }),
						({ makeDiff }) => makeDiff ? 32 : 0
					]
				},
				$$scope: { ctx }
			}
		});

	return {
		c() {
			create_component(makediff.$$.fragment);
		},
		m(target, anchor) {
			mount_component(makediff, target, anchor);
			current = true;
		},
		p(ctx, [dirty]) {
			const makediff_changes = {};

			if (dirty & /*$$scope, makeDiff*/ 40) {
				makediff_changes.$$scope = { dirty, ctx };
			}

			makediff.$set(makediff_changes);
		},
		i(local) {
			if (current) return;
			transition_in(makediff.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(makediff.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(makediff, detaching);
		}
	};
}

function instance$2($$self, $$props, $$invalidate) {
	let { $$slots: slots = {}, $$scope } = $$props;
	let { form } = $$props;
	const dispatch = createEventDispatcher();

	/**
 * Remove a resource from the Form.
 *
 * @type {import('..').remove}
 */
	const remove = makeDiff => ({ id, type }) => {
		delete form.data[id];
		makeDiff(form, id);

		for (let resourceId in form.data) {
			let touched;
			let resource = form.data[resourceId];

			for (let relName of Object.keys(resource.relationships || {})) {
				const rel = resource.relationships[relName];

				if (Array.isArray(rel.data)) {
					let filtered = rel.data.filter(r => r.id !== id || r.type !== type);

					if (filtered.length !== rel.data.length) {
						$$invalidate(1, form.data[resourceId].relationships[relName].data = filtered, form);
						if (!rel.data.length) delete form.data[resourceId].relationships[relName].data;
						touched = true;
					}
				} else if (rel?.data?.id === id && rel?.data?.type === type) {
					delete form.data[resourceId].relationships[relName].data;
					touched = true;
				}

				if (!rel.data) {
					delete form.data[resourceId].relationships[relName];
					touched = true;
				}
			}

			if (!Object.keys(resource.relationships || {}).length) {
				delete form.data[resourceId].relationships;
				touched = true;
			}

			if (touched) makeDiff(form, resourceId);
		}

		dispatch('remove', { id, type });
	};

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(1, form = $$props.form);
		if ('$$scope' in $$props) $$invalidate(3, $$scope = $$props.$$scope);
	};

	return [remove, form, slots, $$scope];
}

class FormRemove extends SvelteComponent {
	constructor(options) {
		super();
		init(this, options, instance$2, create_fragment$2, safe_not_equal, { form: 1 });
	}
}

/* src/Form.svelte generated by Svelte v3.43.0 */

const get_default_slot_changes$1 = dirty => ({
	create: dirty & /*create*/ 256,
	remove: dirty & /*remove*/ 128,
	errors: dirty & /*errors*/ 4,
	disabled: dirty & /*disabled*/ 2
});

const get_default_slot_context$1 = ctx => ({
	create: /*create*/ ctx[8],
	remove: /*remove*/ ctx[7],
	errors: /*errors*/ ctx[2],
	disabled: /*disabled*/ ctx[1]
});

// (12:1) <FormCreate bind:form let:create>
function create_default_slot_1$1(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[3].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[6], get_default_slot_context$1);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, dirty) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope, create, remove, errors, disabled*/ 454)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[6],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[6])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[6], dirty, get_default_slot_changes$1),
						get_default_slot_context$1
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

// (11:0) <FormRemove bind:form let:remove>
function create_default_slot$1(ctx) {
	let formcreate;
	let updating_form;
	let current;

	function formcreate_form_binding(value) {
		/*formcreate_form_binding*/ ctx[4](value);
	}

	let formcreate_props = {
		$$slots: {
			default: [
				create_default_slot_1$1,
				({ create }) => ({ 8: create }),
				({ create }) => create ? 256 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		formcreate_props.form = /*form*/ ctx[0];
	}

	formcreate = new FormCreate({ props: formcreate_props });
	binding_callbacks.push(() => bind(formcreate, 'form', formcreate_form_binding));

	return {
		c() {
			create_component(formcreate.$$.fragment);
		},
		m(target, anchor) {
			mount_component(formcreate, target, anchor);
			current = true;
		},
		p(ctx, dirty) {
			const formcreate_changes = {};

			if (dirty & /*$$scope, create, remove, errors, disabled*/ 454) {
				formcreate_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				formcreate_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			formcreate.$set(formcreate_changes);
		},
		i(local) {
			if (current) return;
			transition_in(formcreate.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(formcreate.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(formcreate, detaching);
		}
	};
}

function create_fragment$1(ctx) {
	let formremove;
	let updating_form;
	let current;

	function formremove_form_binding(value) {
		/*formremove_form_binding*/ ctx[5](value);
	}

	let formremove_props = {
		$$slots: {
			default: [
				create_default_slot$1,
				({ remove }) => ({ 7: remove }),
				({ remove }) => remove ? 128 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		formremove_props.form = /*form*/ ctx[0];
	}

	formremove = new FormRemove({ props: formremove_props });
	binding_callbacks.push(() => bind(formremove, 'form', formremove_form_binding));

	return {
		c() {
			create_component(formremove.$$.fragment);
		},
		m(target, anchor) {
			mount_component(formremove, target, anchor);
			current = true;
		},
		p(ctx, [dirty]) {
			const formremove_changes = {};

			if (dirty & /*$$scope, form, remove, errors, disabled*/ 199) {
				formremove_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				formremove_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			formremove.$set(formremove_changes);
		},
		i(local) {
			if (current) return;
			transition_in(formremove.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(formremove.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(formremove, detaching);
		}
	};
}

function instance$1($$self, $$props, $$invalidate) {
	let errors;
	let disabled;
	let { $$slots: slots = {}, $$scope } = $$props;
	let { form } = $$props;

	function formcreate_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	function formremove_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(0, form = $$props.form);
		if ('$$scope' in $$props) $$invalidate(6, $$scope = $$props.$$scope);
	};

	$$self.$$.update = () => {
		if ($$self.$$.dirty & /*form*/ 1) {
			$$invalidate(2, errors = form?.errors?.other || []);
		}

		if ($$self.$$.dirty & /*form*/ 1) {
			$$invalidate(1, disabled = !form.state || ['saving', 'loading'].includes(form.state));
		}
	};

	return [
		form,
		disabled,
		errors,
		slots,
		formcreate_form_binding,
		formremove_form_binding,
		$$scope
	];
}

class Form extends SvelteComponent {
	constructor(options) {
		super();
		init(this, options, instance$1, create_fragment$1, safe_not_equal, { form: 0 });
	}
}

/* src/Relationship.svelte generated by Svelte v3.43.0 */

const get_default_slot_changes = dirty => ({
	create: dirty & /*create*/ 16777216,
	value: dirty & /*value*/ 64,
	errors: dirty & /*errors*/ 32,
	disabled: dirty & /*disabled*/ 16
});

const get_default_slot_context = ctx => ({
	create: /*create*/ ctx[24],
	set: function func(...args) {
		return /*func*/ ctx[14](/*makeDiff*/ ctx[23], ...args);
	},
	add: function func_1(...args) {
		return /*func_1*/ ctx[15](/*makeDiff*/ ctx[23], ...args);
	},
	remove: function func_2(...args) {
		return /*func_2*/ ctx[16](/*makeDiff*/ ctx[23], ...args);
	},
	value: /*value*/ ctx[6],
	errors: /*errors*/ ctx[5],
	disabled: /*disabled*/ ctx[4]
});

// (85:2) <FieldSetter bind:form {id} {tokens} {debounceMillis} let:set>
function create_default_slot_2(ctx) {
	let current;
	const default_slot_template = /*#slots*/ ctx[13].default;
	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[19], get_default_slot_context);

	return {
		c() {
			if (default_slot) default_slot.c();
		},
		m(target, anchor) {
			if (default_slot) {
				default_slot.m(target, anchor);
			}

			current = true;
		},
		p(ctx, dirty) {
			if (default_slot) {
				if (default_slot.p && (!current || dirty & /*$$scope, create, value, errors, disabled*/ 17301616)) {
					update_slot_base(
						default_slot,
						default_slot_template,
						ctx,
						/*$$scope*/ ctx[19],
						!current
						? get_all_dirty_from_scope(/*$$scope*/ ctx[19])
						: get_slot_changes(default_slot_template, /*$$scope*/ ctx[19], dirty, get_default_slot_changes),
						get_default_slot_context
					);
				}
			}
		},
		i(local) {
			if (current) return;
			transition_in(default_slot, local);
			current = true;
		},
		o(local) {
			transition_out(default_slot, local);
			current = false;
		},
		d(detaching) {
			if (default_slot) default_slot.d(detaching);
		}
	};
}

// (84:1) <FormCreate bind:form let:create>
function create_default_slot_1(ctx) {
	let fieldsetter;
	let updating_form;
	let current;

	function fieldsetter_form_binding(value) {
		/*fieldsetter_form_binding*/ ctx[17](value);
	}

	let fieldsetter_props = {
		id: /*id*/ ctx[1],
		tokens: /*tokens*/ ctx[3],
		debounceMillis: /*debounceMillis*/ ctx[2],
		$$slots: {
			default: [
				create_default_slot_2,
				({ set }) => ({ 25: set }),
				({ set }) => set ? 33554432 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		fieldsetter_props.form = /*form*/ ctx[0];
	}

	fieldsetter = new FieldSetter({ props: fieldsetter_props });
	binding_callbacks.push(() => bind(fieldsetter, 'form', fieldsetter_form_binding));

	return {
		c() {
			create_component(fieldsetter.$$.fragment);
		},
		m(target, anchor) {
			mount_component(fieldsetter, target, anchor);
			current = true;
		},
		p(ctx, dirty) {
			const fieldsetter_changes = {};
			if (dirty & /*id*/ 2) fieldsetter_changes.id = /*id*/ ctx[1];
			if (dirty & /*tokens*/ 8) fieldsetter_changes.tokens = /*tokens*/ ctx[3];
			if (dirty & /*debounceMillis*/ 4) fieldsetter_changes.debounceMillis = /*debounceMillis*/ ctx[2];

			if (dirty & /*$$scope, create, value, errors, disabled*/ 17301616) {
				fieldsetter_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				fieldsetter_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			fieldsetter.$set(fieldsetter_changes);
		},
		i(local) {
			if (current) return;
			transition_in(fieldsetter.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(fieldsetter.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(fieldsetter, detaching);
		}
	};
}

// (83:0) <MakeDiff let:makeDiff>
function create_default_slot(ctx) {
	let formcreate;
	let updating_form;
	let current;

	function formcreate_form_binding(value) {
		/*formcreate_form_binding*/ ctx[18](value);
	}

	let formcreate_props = {
		$$slots: {
			default: [
				create_default_slot_1,
				({ create }) => ({ 24: create }),
				({ create }) => create ? 16777216 : 0
			]
		},
		$$scope: { ctx }
	};

	if (/*form*/ ctx[0] !== void 0) {
		formcreate_props.form = /*form*/ ctx[0];
	}

	formcreate = new FormCreate({ props: formcreate_props });
	binding_callbacks.push(() => bind(formcreate, 'form', formcreate_form_binding));

	return {
		c() {
			create_component(formcreate.$$.fragment);
		},
		m(target, anchor) {
			mount_component(formcreate, target, anchor);
			current = true;
		},
		p(ctx, dirty) {
			const formcreate_changes = {};

			if (dirty & /*$$scope, id, tokens, debounceMillis, form, create, value, errors, disabled*/ 17301631) {
				formcreate_changes.$$scope = { dirty, ctx };
			}

			if (!updating_form && dirty & /*form*/ 1) {
				updating_form = true;
				formcreate_changes.form = /*form*/ ctx[0];
				add_flush_callback(() => updating_form = false);
			}

			formcreate.$set(formcreate_changes);
		},
		i(local) {
			if (current) return;
			transition_in(formcreate.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(formcreate.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(formcreate, detaching);
		}
	};
}

function create_fragment(ctx) {
	let makediff;
	let current;

	makediff = new MakeDiff({
			props: {
				$$slots: {
					default: [
						create_default_slot,
						({ makeDiff }) => ({ 23: makeDiff }),
						({ makeDiff }) => makeDiff ? 8388608 : 0
					]
				},
				$$scope: { ctx }
			}
		});

	return {
		c() {
			create_component(makediff.$$.fragment);
		},
		m(target, anchor) {
			mount_component(makediff, target, anchor);
			current = true;
		},
		p(ctx, [dirty]) {
			const makediff_changes = {};

			if (dirty & /*$$scope, form, id, tokens, debounceMillis, value, errors, disabled*/ 524415) {
				makediff_changes.$$scope = { dirty, ctx };
			}

			makediff.$set(makediff_changes);
		},
		i(local) {
			if (current) return;
			transition_in(makediff.$$.fragment, local);
			current = true;
		},
		o(local) {
			transition_out(makediff.$$.fragment, local);
			current = false;
		},
		d(detaching) {
			destroy_component(makediff, detaching);
		}
	};
}

function instance($$self, $$props, $$invalidate) {
	let tokens;
	let value;
	let errors;
	let disabled;
	let { $$slots: slots = {}, $$scope } = $$props;
	let { form } = $$props;
	let { id } = $$props;
	let { name } = $$props;
	let { type } = $$props;
	let { isArray } = $$props;
	let { debounceMillis } = $$props;

	let preUpdate = arrayOps => {
		if (isArray && !arrayOps || !isArray && arrayOps) {
			$$invalidate(0, form.errors = form.errors || { other: [], mapped: {} }, form);
			$$invalidate(0, form.errors.other = form.errors.other || [], form);

			form.errors.other.push({
				code: 'Incorrectly Designed Form',
				detail: `"Relationship" component attempted "onSet" but was configured as an array. [id=${id},name=${name}]`,
				meta: { formErrorId: id }
			});

			return true;
		} else {
			if (!form.data[id]?.relationships) $$invalidate(0, form.data[id].relationships = {}, form);
			if (!form.data[id]?.relationships[name]) $$invalidate(0, form.data[id].relationships[name] = {}, form);
			if (!form.data[id]?.relationships[name].data) $$invalidate(0, form.data[id].relationships[name].data = isArray ? [] : {}, form);
		}
	};

	let cleanup = () => {
		if (form.data[id]?.relationships[name]?.data?.length === 0 || !form.data[id]?.relationships[name]?.data) delete form.data[id].relationships[name];
		if (Object.keys(form.data[id]?.relationships).length === 0) delete form.data[id].relationships;
	};

	const dispatch = createEventDispatcher();

	let onSet = (value, makeDiff) => {
		if (value) {
			preUpdate();
			$$invalidate(0, form.data[id].relationships[name] = { data: { id: value, type } }, form);
		} else if (form.data[id].relationships[name]) {
			$$invalidate(0, form.data[id].relationships[name] = undefined, form);
		}

		cleanup();
		makeDiff(form, id);
		dispatch('change', { id, keypath: tokens, value });
	};

	let onAdd = (value, makeDiff) => {
		preUpdate(true);
		$$invalidate(0, form.data[id].relationships[name].data = [...form.data[id].relationships[name].data, { id: value, type }], form);
		makeDiff(form, id);
	};

	let onRemove = (value, makeDiff) => {
		preUpdate(true);

		$$invalidate(
			0,
			form.data[id].relationships[name].data = [
				...form.data[id].relationships[name].data.filter(rel => !(rel.id === value && rel.type === type))
			],
			form
		);

		cleanup();
		makeDiff(form, id);
	};

	const func = (makeDiff, value) => onSet(value, makeDiff);
	const func_1 = (makeDiff, value) => onAdd(value, makeDiff);
	const func_2 = (makeDiff, value) => onRemove(value, makeDiff);

	function fieldsetter_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	function formcreate_form_binding(value) {
		form = value;
		$$invalidate(0, form);
	}

	$$self.$$set = $$props => {
		if ('form' in $$props) $$invalidate(0, form = $$props.form);
		if ('id' in $$props) $$invalidate(1, id = $$props.id);
		if ('name' in $$props) $$invalidate(10, name = $$props.name);
		if ('type' in $$props) $$invalidate(11, type = $$props.type);
		if ('isArray' in $$props) $$invalidate(12, isArray = $$props.isArray);
		if ('debounceMillis' in $$props) $$invalidate(2, debounceMillis = $$props.debounceMillis);
		if ('$$scope' in $$props) $$invalidate(19, $$scope = $$props.$$scope);
	};

	$$self.$$.update = () => {
		if ($$self.$$.dirty & /*name*/ 1024) {
			$$invalidate(3, tokens = ['relationships', name, 'data']);
		}

		if ($$self.$$.dirty & /*form, id, tokens*/ 11) {
			$$invalidate(6, value = get(form.data[id], tokens) || '');
		}

		if ($$self.$$.dirty & /*form, id, tokens*/ 11) {
			$$invalidate(5, errors = get(form, ['errors', 'mapped', id, ...tokens]) || []);
		}

		if ($$self.$$.dirty & /*form*/ 1) {
			$$invalidate(4, disabled = !form.state || ['saving', 'loading'].includes(form.state));
		}
	};

	return [
		form,
		id,
		debounceMillis,
		tokens,
		disabled,
		errors,
		value,
		onSet,
		onAdd,
		onRemove,
		name,
		type,
		isArray,
		slots,
		func,
		func_1,
		func_2,
		fieldsetter_form_binding,
		formcreate_form_binding,
		$$scope
	];
}

class Relationship extends SvelteComponent {
	constructor(options) {
		super();

		init(this, options, instance, create_fragment, safe_not_equal, {
			form: 0,
			id: 1,
			name: 10,
			type: 11,
			isArray: 12,
			debounceMillis: 2
		});
	}
}

exports.Field = Field;
exports.FieldSetter = FieldSetter;
exports.Form = Form;
exports.FormCreate = FormCreate;
exports.FormRemove = FormRemove;
exports.MakeDiff = MakeDiff;
exports.Relationship = Relationship;
